<?php

namespace Drupal\consent\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class ConsentLayerBlockBase.
 */
abstract class ConsentLayerBlockBase extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    $libraries = ['consent/layer'];
    foreach ($this->configuration['trigger'] as $trigger => $enabled) {
      if ($enabled) {
        $libraries[] = 'consent/trigger.' . $trigger;
      }
    }
    return [
      '#attached' => [
        'library' => $libraries,
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    return parent::blockForm($form, $form_state) + [
      'trigger' => [
        '#type' => 'fieldset',
        '#title' => $this->t('Trigger for user consents'),
        '#collapsible' => FALSE,
        '#collapsed' => FALSE,
      ] + $this->buildTriggerElements($form, $form_state),
    ];
  }

  /**
   * Build the trigger form elements.
   *
   * @param array $form
   *   The main form build array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The trigger form elements.
   */
  protected function buildTriggerElements(array $form, FormStateInterface $form_state) {
    $elements = [];
    foreach ($this->getConsentTriggers() as $trigger => $label) {
      $elements[$trigger] = [
        '#type' => 'checkbox',
        '#title' => $this->t($label),
        '#default_value' => $this->configuration['trigger'][$trigger],
      ];
    }
    return $elements;
  }

  /**
   * Get all available consent triggers.
   *
   * @return array
   *   The consent triggers.
   */
  protected function getConsentTriggers() {
    return [
      'storage' => 'Submit user consents to the backend storage.',
      'click' => 'Enable automatic opt-in when clicking.',
      'scroll' => 'Enable automatic opt-in when scrolling.',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValue('trigger', []);
    $triggers = $this->getConsentTriggers();
    foreach (array_keys($triggers) as $trigger) {
      $triggers[$trigger] = !empty($values[$trigger]);
    }
    $this->configuration['trigger'] = $triggers;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $default = parent::defaultConfiguration();
    foreach (array_keys($this->getConsentTriggers()) as $trigger) {
      $default['trigger'][$trigger] = FALSE;
    }
    return $default;
  }

}
